/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.binding.xmpp;

import javax.jbi.JBIException;

import org.objectweb.petals.binding.xmpp.listeners.SessionDescriptor;
import org.objectweb.petals.binding.xmpp.listeners.SessionDescriptorBuilder;
import org.objectweb.petals.binding.xmpp.listeners.XmppBCExternalListener;
import org.objectweb.petals.binding.xmpp.listeners.XmppBCJBIListener;
import org.objectweb.petals.binding.xmpp.listeners.XmppSessionManager;
import org.objectweb.petals.binding.xmpp.listeners.XmppMessageManager;
import org.objectweb.petals.component.framework.bc.AbstractBindingComponent;

/**
 * The Xmpp Binding Component.
 * <p>
 * It creates some sub components like :
 * <p>
 * the <b>{@link XmppSessionManager}</b>, used to manage connections with
 * jabber server
 * <p>
 * the <b>{@link XmppMessageManager}</b>, used to process (reading or
 * creating) xmpp messages
 * <p>
 * the <b>{@link SessionDescriptorBuilder}</b>, used to build a <b>{@link SessionDescriptor}</b>
 * which contains informations about user session
 * <p>
 * the <b>{@link XmppBCExternalListener}</b> used to manage listeners for
 * external xmpp messages
 * <p>
 * the <b>{@link XmppBCJBIListener}</b> used to process incoming JBI messages
 * <p>
 * This class also defines some Default Values for xmpp features like :
 * <p>
 * <b>the default server port (5222),
 * <p>
 * <b>the default transfert timeout in ms (5222),
 * <p>
 * <b>the default session timeout in ms (300000)
 * <p>
 * 
 * @author mcarpentier - EBM Websourcing
 * 
 */
public class XmppBCImpl extends AbstractBindingComponent {

    public static final String XMPP_PORT_DEFAULT = "5222";

    // defaut session timeout in ms : if no message is sent for this period,
    // JBIListener xmpp session will be ended
    public static final String SESSION_TIMEOUT_DEFAULT = "30000";

    public static final String SESSION_TIMEOUT_PATHELEMENT = "session-timeout";

    // defaut transfert timeout in ms
    public static final String TRANSFERT_TIMEOUT_DEFAULT = "60000";

    public static final String TRANSFERT_TIMEOUT_PATHELEMENT = "transfert-timeout";

    public static final String PACKETS_PATHELEMENT = "packets";

    public static final String PACKETS_DEFAULT = "all";

    public static final String USERNAME_PATHELEMENT = "username";

    public static final String PASSWORD_PATHELEMENT = "password";

    public static final String RESOURCE_PATHELEMENT = "resource";

    public static final String PORT_PATHELEMENT = "port";

    public static final String HOSTNAME_PATHELEMENT = "hostname";

    public static final String DEFAULT_DESTINATION_PATHELEMENT = "default-destination";

    public static final String OPERATION_PATHELEMENT = "operation";

    public static final String DESTINATION_ADDRESS = "destinationAddress";

    public static final String FROM_ADDRESS = "fromAddress";

    public static final String ROOM_PASSWORD = "roomPassword";

    public static final String DESTINATION_ROOM = "destinationRoom";

    public static final String ENCODING_PATHELEMENT = "encoding";

    public static final String DECODING_PATHELEMENT = "decoding";

    public static final String REDIRECTION_PATHELEMENT = "redirection";

    public static final String AVATAR_FILE = "avatar.jpg";

    public static final String PETALS_AVATAR_FILE = "petals-avatar.jpg";
    
    public static final String UTF8_CHARFORMAT="UTF-8";

    protected SessionDescriptorBuilder sessionDescriptorBuilder;

    protected XmppSessionManager xmppConnectionManager;

    protected XmppMessageManager xmppMessageManager;

    /**
     * The XMPP component context
     */
    protected XmppComponentContext xmppContext;

    @Override
    protected void doInit() throws JBIException {

        this.xmppConnectionManager = new XmppSessionManager(this.getLogger(),
                this.getDefaultPath());

        this.sessionDescriptorBuilder = new SessionDescriptorBuilder(this
                .getLogger());

        this.xmppMessageManager = new XmppMessageManager(this.getLogger(), this);

        this.xmppContext = new XmppComponentContext();

        // set service unit listener
        XmppSUListener suListener = new XmppSUListener(
                this.xmppConnectionManager, this.xmppContext, this.getLogger());
        this.setServiceUnitListener(suListener);

    }

    public SessionDescriptorBuilder getSessionDescriptorBuilder() {
        return this.sessionDescriptorBuilder;
    }

    public XmppSessionManager getXmppConnectionManager() {
        return this.xmppConnectionManager;
    }

    public XmppMessageManager getXmppMessageManager() {
        return this.xmppMessageManager;
    }

    /**
     * Get the default path to copy files to.
     * 
     * @return
     */
    public String getDefaultPath() {
        String baseDir = this.getContext().getInstallRoot();
        StringBuffer sb = new StringBuffer(baseDir.substring(0, baseDir
                .length()
                - "install".length()));
        sb.append("XmppBC");
        return sb.toString();
    }

    /**
     * Get the default path to resource files.
     * 
     * @return
     */
    public String getDefaultResourcesPath() {
        String baseDir = this.getContext().getInstallRoot();
        return baseDir;
    }

    /**
     * close all current connections to jabber server
     * 
     */
    @Override
    protected void doStop() throws JBIException {
        if (this.xmppConnectionManager.getXmppSessions() != null) {
            for (SessionDescriptor session : this.xmppConnectionManager
                    .getXmppSessions().values()) {
                if ((session.getXmppConnection() != null)
                        && (session.getXmppConnection().isConnected())) {
                    session.getXmppConnection().disconnect();
                }
            }
        }
    }

    public XmppComponentContext getXmppContext() {
        return this.xmppContext;
    }

}
