/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.binding.xmpp.listeners;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Map;
import java.util.Properties;
import java.util.logging.Logger;

import org.jivesoftware.smackx.packet.VCard;
import org.objectweb.petals.binding.xmpp.XmppBCImpl;
import org.objectweb.petals.component.framework.PEtALSCDKException;
import org.objectweb.petals.component.framework.util.Extensions;
import org.objectweb.petals.component.framework.util.StringHelper;

/**
 * This class is used to build a session descriptor from service unit descriptor
 * @author mcarpentier - EBM Websourcing
 */
public class SessionDescriptorBuilder {

    /**
     * file suffix for URL() constructor
     */
    protected static final String FILE_URL_SUFFIX = "file://";

    /**
     * logger
     */
    protected Logger log;

    /**
     * @param log
     *            the component logger
     */
    public SessionDescriptorBuilder(final Logger log) {
        super();
        this.log = log;
    }

    /**
     * Retrieve xmpp connection properties from the service unit descriptor
     * Descriptor default values : <br />
     * hostname : the hostname, non null and non empty<br />
     * port : the port, default value : "5222"<br />
     * username : the username, default value : no default<br />
     * password : the password, default value : no default<br />
     * resource : the connection resource, default value : no default<br />
     * toAddress : the destination address, can be null or empty<br />
     * transfert-timeout : the timeout for any file transfert in ms, default
     * value : "60000"<br />
     * @param extensions
     *            extensions associated to the given message exchange
     * @return a <b>{@link SessionDescriptor}</b> filled in with extracted
     *         information
     * @throws PEtALSCFException
     *             if properties checking fails
     */
    public final SessionDescriptor build(final Extensions extensions)
        throws PEtALSCDKException {
        final SessionDescriptor sessionDescriptor = new SessionDescriptor();
        final Properties extensionsProps = extensions.getPetalsExtensions();

        // Retrieve user and password
        final String username = extensionsProps
                .getProperty(XmppBCImpl.USERNAME_PATHELEMENT);
        final String resource = extensionsProps
                .getProperty(XmppBCImpl.RESOURCE_PATHELEMENT);
        final String defaultDestination = extensionsProps
                .getProperty(XmppBCImpl.DEFAULT_DESTINATION_PATHELEMENT);
        final SessionUser user = new SessionUser();
        user.setUsername(username);
        user.setPassword(extensionsProps
                .getProperty(XmppBCImpl.PASSWORD_PATHELEMENT));
        user.setResource(resource);
        user.setDefaultDestination(defaultDestination);
        sessionDescriptor.setUser(user);

        // Retrieve host and port
        final String host = extensionsProps
                .getProperty(XmppBCImpl.HOSTNAME_PATHELEMENT);
        sessionDescriptor.setHostname(host);
        sessionDescriptor.setPort(extensionsProps.getProperty(
                XmppBCImpl.PORT_PATHELEMENT, XmppBCImpl.XMPP_PORT_DEFAULT));

        // Set user's server id : <username>@<host>/<resource>
        String tmpUsername = "";
        String tmpHost = "";
        String tmpResource = "";
        if (!(StringHelper.isNullOrEmpty(username))) {
            tmpUsername = username;
        }
        if (!(StringHelper.isNullOrEmpty(host))) {
            tmpHost = "@" + host;
        }
        if (!(StringHelper.isNullOrEmpty(resource))) {
            tmpResource = "/" + resource;
        }
        user.setid(tmpUsername + tmpHost + tmpResource);

        String sessionTimeout = extensionsProps
                .getProperty(XmppBCImpl.SESSION_TIMEOUT_PATHELEMENT);

        // if no prop found, set to the default value
        if (StringHelper.isNullOrEmpty(sessionTimeout)) {
            sessionTimeout = XmppBCImpl.SESSION_TIMEOUT_DEFAULT;
        }

        sessionDescriptor.setSessionTimeout(sessionTimeout);

        // Check mandatory properties
        this.checkProperties(sessionDescriptor);

        return sessionDescriptor;
    }

    /**
     * Retrieve user's avatar and set it to session's Visit Card
     * @param sd
     *            the {@link SessionDescriptor} of the xmpp connection
     * @param pathsMap
     *            a map of String with root paths for all the SU
     * @param defaultAvatar
     *            the path of the default XmppBC avatar
     * @throws PEtALSCDKException
     *             if an error occurs during the loading of the picture
     */
    public final void updateUserVcard(final SessionDescriptor sd,
            final Map<String, String> pathsMap, final String defaultAvatar)
        throws PEtALSCDKException {
        if ((pathsMap != null) && (sd.getUser() != null)) {
            final String rootPath = pathsMap.get(sd.getUser().getUsername()
                    + "@" + sd.getHostname());
            final VCard vCard = new VCard();
            URL avatarURL;
            try {
                File f = new File(rootPath + File.separator
                        + XmppBCImpl.AVATAR_FILE);
                if (f.exists()) {
                    avatarURL = new URL(SessionDescriptorBuilder.FILE_URL_SUFFIX + rootPath
                            + File.separator + XmppBCImpl.AVATAR_FILE);
                    vCard.setAvatar(avatarURL);
                    sd.getUser().setVCard(vCard);
                } else {
                    f = new File(defaultAvatar);
                    if (f.exists()) {
                        avatarURL = new URL(SessionDescriptorBuilder.FILE_URL_SUFFIX + defaultAvatar);
                        vCard.setAvatar(avatarURL);
                        sd.getUser().setVCard(vCard);
                    }
                }
            } catch (final MalformedURLException e) {
                final String msg = "Can't open required file";
                this.log.severe(msg);
                throw new PEtALSCDKException(msg);
            }
        }

    }

    /**
     * Complete xmpp session properties from the provide service unit descriptor
     * Descriptor values : <br />
     * session-timeout : the timeout for any file transfert in ms, default value :
     * "300000"<br />
     * @param sd
     *            The descriptor of the session
     * @param extensions
     *            extensions associated to the given message exchange
     * @throws PEtALSCDKException
     *             if an error occurs
     */
    public final void completeWithProvideProperties(final SessionDescriptor sd,
            final Extensions extensions) throws PEtALSCDKException {
        if (sd != null) {
            final Properties extensionsProps = extensions.getPetalsExtensions();
            final String defaultDestination = extensionsProps
                    .getProperty(XmppBCImpl.DEFAULT_DESTINATION_PATHELEMENT);

            if (sd.getUser() != null) {
                sd.getUser().setDefaultDestination(defaultDestination);
            }

            // Retrieve timeouts
            String transfertTimeout = extensionsProps
                    .getProperty(XmppBCImpl.TRANSFERT_TIMEOUT_PATHELEMENT);

            // if the property hasn't been found, set to the default value
            if (StringHelper.isNullOrEmpty(transfertTimeout)) {
                transfertTimeout = XmppBCImpl.TRANSFERT_TIMEOUT_DEFAULT;
            }

            // Retrieve encoding format
            final String encoding = extensionsProps
                    .getProperty(XmppBCImpl.ENCODING_PATHELEMENT);

            sd.setEncoding(encoding);
            sd.setTransfertTimeout(transfertTimeout);

        }

    }

    /**
     * Complete xmpp session properties from the consume service unit descriptor
     * Descriptor values : <br />
     * session-timeout : the timeout for any file transfert in ms, default value :
     * "300000"<br />
     * @param sd
     *            The descriptor of the session
     * @param extensions
     *            extensions associated to the given message exchange
     * @return a <b>{@link SessionDescriptor}</b> filled in with extracted
     *         information
     * @throws PEtALSCDKException
     *             if an error occurs
     */
    public final void completeWithConsumeProperties(final SessionDescriptor sd,
            final Extensions extensions) throws PEtALSCDKException {
        if (sd != null) {
            final Properties extensionsProps = extensions.getPetalsExtensions();
            // Retrieve timeouts
            String transfertTimeout = extensionsProps
                    .getProperty(XmppBCImpl.TRANSFERT_TIMEOUT_PATHELEMENT);

            // if the property hasn't been found, set to the default value
            if (StringHelper.isNullOrEmpty(transfertTimeout)) {
                transfertTimeout = XmppBCImpl.TRANSFERT_TIMEOUT_DEFAULT;
            }

            // Retrieve timeouts
            final String operation = extensionsProps
                    .getProperty(XmppBCImpl.OPERATION_PATHELEMENT);

            // Retrieve decoding format
            final String decoding = extensionsProps
                    .getProperty(XmppBCImpl.DECODING_PATHELEMENT);

            // Retrieve redirection
            final String redirection = extensionsProps
                    .getProperty(XmppBCImpl.REDIRECTION_PATHELEMENT);

            // Retrieve packets listening
            final String packets = extensionsProps
                    .getProperty(XmppBCImpl.PACKETS_PATHELEMENT);

            sd.setDecoding(decoding);

            sd.setTransfertTimeout(transfertTimeout);
            sd.setOperation(operation);
            sd.setRedirection(redirection);
            sd.setPackets(packets);

        }

    }

    /**
     * Check the properties extracted from the service unit descriptor All
     * properties are mandatory except port
     * @param sessionDescriptor the session descriptor
     * @throws PEtALSCDKException if an error occurs
     */
    protected final void checkProperties(
            final SessionDescriptor sessionDescriptor) throws PEtALSCDKException {
        String msg = "";
        if (StringHelper.isNullOrEmpty(sessionDescriptor.getHostname())) {
            msg = "The host name of the requested external "
                    + "service must be specified";
            this.log.severe(msg);
            throw new PEtALSCDKException(msg);
        }
        final SessionUser user = sessionDescriptor.getUser();
        if ((user == null) || (StringHelper.isNullOrEmpty(user.getUsername()))
                || (StringHelper.isNullOrEmpty(user.getPassword()))) {
            msg = "Username and password must be specified ";
            this.log.severe(msg);
            throw new PEtALSCDKException(msg);
        }
    }
}
