/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.binding.xmpp.listeners;

import java.io.File;
import java.util.Map;
import java.util.logging.Level;

import javax.jbi.JBIException;
import javax.jbi.messaging.MessagingException;

import org.jivesoftware.smack.XMPPConnection;
import org.jivesoftware.smack.filter.PacketFilter;
import org.jivesoftware.smack.filter.PacketTypeFilter;
import org.jivesoftware.smack.packet.Message;
import org.objectweb.petals.binding.xmpp.XmppBCImpl;
import org.objectweb.petals.component.framework.PEtALSCDKException;
import org.objectweb.petals.component.framework.listener.AbstractExternalListener;
import org.objectweb.petals.component.framework.util.Exchange;
import org.objectweb.petals.component.framework.util.Extensions;
import org.objectweb.petals.component.framework.util.StringHelper;

/**
 * This class is used to manage listeners for external incoming messages sent to
 * registered users connected during SU deployments
 * 
 * @author mcarpentier - EBM Websourcing
 * 
 */
public class XmppBCExternalListener extends AbstractExternalListener {

    protected SessionDescriptorBuilder sessionDescriptorBuilder;

    protected XmppSessionManager xmppConnectionManager;

    private XmppMessageManager xmppMessageManager;

    private SessionDescriptor sessionDescriptor;

    /**
     * constructor
     */
    public XmppBCExternalListener() {
        super();
    }

    @Override
    protected final void init() {
        this.sessionDescriptorBuilder = ((XmppBCImpl) this.getComponent())
                .getSessionDescriptorBuilder();

        this.xmppConnectionManager = ((XmppBCImpl) this.getComponent())
                .getXmppConnectionManager();

        this.xmppMessageManager = ((XmppBCImpl) this.getComponent())
                .getXmppMessageManager();
    }

    /**
     * init connection with SU descriptor
     * 
     * @throws PEtALSCDKException
     *             if and error occurs
     * 
     */
    @Override
    protected final void start() throws PEtALSCDKException {
        // Extract session information from SU extensions
        final Extensions extensions = new Extensions(this.getConsumes()
                .getExtensions());

        try {
            // Retrieve or init the xmpp session
            this.sessionDescriptor = this.xmppConnectionManager
                    .retrieveXmppSessionFromExtensions(extensions);

            if (this.sessionDescriptor == null) {
                this.sessionDescriptor = this.sessionDescriptorBuilder
                        .build(extensions);

                if ((this.sessionDescriptor.getUser() == null)
                        || (StringHelper.isNullOrEmpty(this.sessionDescriptor
                                .getUser().getid()))) {
                    this.getLogger().log(Level.SEVERE,
                            "Couldn't init session - User is null");
                }
                this.xmppConnectionManager.getXmppSessions().put(
                        this.sessionDescriptor.getUser().getid(),
                        this.sessionDescriptor);

            }

            // deals with VCard information
            if (((XmppBCImpl) this.getComponent()).getXmppContext() != null) {
                final Map<String, String> map = ((XmppBCImpl) this
                        .getComponent()).getXmppContext().getSuRootPaths();
                final String defaultAvatar = ((XmppBCImpl) this.getComponent())
                        .getDefaultResourcesPath()
                        + File.separator + XmppBCImpl.PETALS_AVATAR_FILE;
                this.sessionDescriptorBuilder.updateUserVcard(
                        this.sessionDescriptor, map, defaultAvatar);

            }

            // load specific consume SU properties
            this.sessionDescriptorBuilder.completeWithConsumeProperties(
                    this.sessionDescriptor, extensions);

            this.xmppConnectionManager.connectSession(this.sessionDescriptor);

            // enable VCard support
            this.xmppConnectionManager.handleVCard(this.sessionDescriptor);

            // start listening for incoming xmpp messages
            this.startListening(this.sessionDescriptor);

        } catch (final JBIException e) {
            final String msg = "Unable to connect to server";
            this.getLogger().log(Level.SEVERE, msg, e);
            throw new PEtALSCDKException(msg, e);
        }

    }

    /**
     * disconnect the current connection on stop
     * 
     * @throws PEtALSCDKException
     *             if an error occurs
     */
    @Override
    protected final void stop() throws PEtALSCDKException {
        if ((this.sessionDescriptor != null)
                && (this.sessionDescriptor.getXmppConnection() != null)) {
            final XMPPConnection connection = this.sessionDescriptor
                    .getXmppConnection();
            if ((connection != null) && (connection.isConnected())) {
                connection.disconnect();
                this.sessionDescriptor.setListening(false);
            }

        }

    }

    protected final void sendPacket(final Exchange mew)
        throws PEtALSCDKException {
        try {
            this.sendSync(mew);
        } catch (final MessagingException e) {
            final String msg = "Error while sending packet";
            this.getLogger().log(Level.SEVERE, msg, e);
            throw new PEtALSCDKException(msg, e);
        }
    }

    /**
     * Init a listener on the given connection to listen to incoming text
     * messages {@link Message} Other message types are ignored
     * 
     * @param sd
     *            the xmpp connection's descriptor
     * 
     */
    protected final void startListening(final SessionDescriptor sd) {

        final String operation = sd.getOperation();
        final String redirection = sd.getRedirection();
        final String packets = sd.getPackets();

        this.createPacketListener(sd.getXmppConnection(), redirection,
                operation, packets);

        sd.setListening(true);
    }

    /**
     * Add a global packet listener to the given {@link XMPPConnection}
     * 
     * @param connection
     *            connection to add the listener
     * @param redirection
     *            jabber JID to send packets to.If null or empty, take packet's
     *            destionation
     * @param operation
     *            operation for the newly created Message Exchange
     * @param packets
     *            the packet type to listen for (default : ALL)
     */
    protected final void createPacketListener(final XMPPConnection connection,
            final String redirection, final String operation,
            final String packets) {
        PacketFilter filter = null;
        if ((!(StringHelper.isNullOrEmpty(packets)))
                && (packets.indexOf("Message") != -1)) {
            this.getLogger().info(
                    "Starting listening for incoming Message packets");
            filter = new PacketTypeFilter(Message.class);
        } else {
            this.getLogger().info("Starting listening for incoming packets");
        }
        final XmppBCPacketListener myListener = new XmppBCPacketListener(
                this.sessionDescriptor, this, redirection, operation, this
                        .getLogger());
        // Register the listener.
        if (connection != null) {
            connection.addPacketListener(myListener, filter);
        }
    }

    public final XmppMessageManager getXmppMessageManager() {
        return this.xmppMessageManager;
    }

}
