/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.binding.xmpp.listeners;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.logging.Logger;

import javax.activation.DataHandler;
import javax.jbi.JBIException;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.namespace.QName;
import javax.xml.transform.Source;

import org.jivesoftware.smack.packet.Message;
import org.jivesoftware.smack.packet.Packet;
import org.objectweb.petals.binding.xmpp.XmppBCImpl;
import org.objectweb.petals.binding.xmpp.util.XmppUtil;
import org.objectweb.petals.component.framework.MEPConstants;
import org.objectweb.petals.component.framework.PEtALSCDKException;
import org.objectweb.petals.component.framework.bc.AbstractBindingComponent;
import org.objectweb.petals.component.framework.util.Exchange;
import org.objectweb.petals.component.framework.util.SourceHelper;
import org.objectweb.petals.component.framework.util.StringHelper;

/**
 * This class is used to process (reading or creating) xmpp messages
 * 
 * @author mcarpentier - EBM Websourcing
 */
public class XmppMessageManager {

    /**
     * logger
     */
    protected Logger log;

    /**
     * @param log
     *            the component {@link Logger}
     * @param component
     *            the component that instanciate this manager
     */
    public XmppMessageManager(final Logger log,
            final AbstractBindingComponent component) {
        super();
        this.log = log;
    }

    /**
     * Create a {@link Message} from the given {@link NormalizedMessage}
     * 
     * @param exchange
     *            the {@link MessageExchangeWrapper}
     * @param sessionDescriptor
     *            user's session descriptor
     * @return the created {@link Message}
     * @throws JBIException
     *             if an error occurs during {@link Message} creation
     */
    public final Message mapNormalizedMessageToXmppMessage(
            final Exchange exchange, final SessionDescriptor sessionDescriptor)
            throws JBIException {

        final Message msg = new Message();
        final NormalizedMessage inMsg = exchange.getInMessage();
        if (inMsg == null) {
            final String m = "Error while processing message : message is null";
            throw new JBIException(m);
        }
        String content = null;
        String dest = null;
        if (inMsg.getContent() != null) {
            // retrieve content as text
            content = SourceHelper.createURLDecodedString(inMsg.getContent(),
                    XmppBCImpl.UTF8_CHARFORMAT);

            content = XmppUtil.getInstance().deleteRootTag(content);
        }
        dest = (String) exchange.getProperty(XmppBCImpl.DESTINATION_ADDRESS);

        // set mesage's text
        msg.setBody(content);

        if ((StringHelper.isNullOrEmpty(dest))
                && (sessionDescriptor.getUser() != null)) {
            dest = sessionDescriptor.getUser().getDefaultDestination();
        }

        // set message's destination, if not found in message retrieve it form
        // service unit descriptor
        msg.setTo(dest);

        // add all properties to the message exchange
        for (Object name : exchange.getPropertyNames()) {
            final String nameString = (String) name;
            msg.setProperty(nameString, exchange.getProperty(nameString));
        }

        return msg;
    }

    /**
     * Create a {@link MessageExchange} from a xmpp packet {@link Packet} and
     * send it to the component linked to the given address.
     * 
     * @param listener
     *            the xmmp external listener
     * @param packet
     *            the {@link Packet} to process
     * @param operation
     *            the service operation to invoke
     * @param constant
     *            the mail {@link Message} to process
     * @param redirection
     *            the optional address to automatically redirect messages to
     * @return a newly created {@link Exchange}
     * @throws JBIException
     *             if an error occurs during parameter retrieval from subject,
     *             during {@link MessageExchange} creation or during
     *             {@link MessageExchange} sending
     */
    public final Exchange process(final XmppBCExternalListener listener,
            final String redirection, final Packet packet,
            final String operation, final MEPConstants constant)
            throws JBIException {

        Exchange ex = null;
        final Map<String, String> props = new HashMap<String, String>();
        if (!(StringHelper.isNullOrEmpty(redirection))) {
            props.put(XmppBCImpl.DESTINATION_ADDRESS, redirection);
        } else {
            props.put(XmppBCImpl.DESTINATION_ADDRESS, packet.getTo());
        }
        props.put(XmppBCImpl.FROM_ADDRESS, packet.getFrom());

        final Set<DataHandler> attachments = new HashSet<DataHandler>();

        try {
            if (packet instanceof Message) {
                final Message msg = (Message) packet;
                this.log.info("Message : " + msg.getBody());
                if (!(StringHelper.isNullOrEmpty(msg.getBody()))) {
                    ex = this.createJBIMessageExchange(listener, operation,
                            constant, msg.getBody(), attachments, props);
                }
            } else {
                ex = this.createJBIMessageExchange(listener, operation,
                        constant, packet.toXML(), attachments, props);
            }
        } catch (final MessagingException e) {
            final String m = "Error during creation of the JBI "
                    + "message exchange from mail information";
            throw new JBIException(m, e);
        }

        return ex;
    }

    /**
     * Create a new Message exchange with the provided information.
     * 
     * @param constant
     *            the message exchange pattern
     * @param listener
     *            the xmmp external listener
     * @param jbiContent
     *            the String content to send
     * @param operation
     *            the service operation to invoke
     * @param attachments
     *            a set of attachments
     * @param properties
     *            the {@link MessageExchange}
     * @return a newly created {@link Exchange}
     * @throws MessagingException
     *             if an error occurs
     */
    protected final Exchange createJBIMessageExchange(
            final XmppBCExternalListener listener, final String operation,
            final MEPConstants constant, final String jbiContent,
            final Set<DataHandler> attachments,
            final Map<String, String> properties) throws MessagingException {
        final Exchange ex = listener.createConsumeExchange(constant);
        this.completeExchange(ex, operation, jbiContent, attachments,
                properties);
        return ex;
    }

    /**
     * fill the given {@link Exchange} it with the given information (content
     * and attachments).
     * 
     * @param ex
     *            the Message Exchange
     * @param operation
     *            the service operation to invoke the {@link NormalizedMessage}
     *            content
     * @param jbiContent
     *            the message content
     * @param attachments
     *            the {@link NormalizedMessage} attachments
     * @param properties
     *            the {@link MessageExchange}
     * @throws javax.jbi.messaging.MessagingException
     *             if an error occured during {@link MessageExchange} creation
     *             and filling
     */
    protected final void completeExchange(final Exchange ex,
            final String operation, final String jbiContent,
            final Set<DataHandler> attachments,
            final Map<String, String> properties)
            throws javax.jbi.messaging.MessagingException {

        // Create the "in" normalized message
        final NormalizedMessage nm = ex.getInMessage();
        // Fill content
        Source source;

        try {
            source = SourceHelper.createSourceWithURLEncoding(jbiContent,
                    "UTF-8");

        } catch (final PEtALSCDKException e) {
            throw new javax.jbi.messaging.MessagingException(
                    "Can't create source from String content", e);
        }
        nm.setContent(source);

        // Fill attachments
        if ((attachments != null) && (!attachments.isEmpty())) {
            for (DataHandler handler : attachments) {
                nm.addAttachment(handler.getName(), handler);
            }
        }

        // Set Operation
        if (!StringHelper.isNullOrEmpty(operation)) {
            ex.setOperation(new QName(operation));
        }

        // Set properties
        for (final Iterator iter = properties.entrySet().iterator(); iter
                .hasNext();) {
            final Map.Entry entry = (Map.Entry) iter.next();
            final String key = (String) entry.getKey();
            final String value = (String) entry.getValue();
            if ((!StringHelper.isNullOrEmpty(key))
                    && (!StringHelper.isNullOrEmpty(value))) {
                ex.setProperty(key, value);
            }
        }

    }

}
