/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.binding.xmpp.listeners;

import java.io.File;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.messaging.NormalizedMessage;

import org.jivesoftware.smack.Chat;
import org.jivesoftware.smack.ChatManager;
import org.jivesoftware.smack.ConnectionConfiguration;
import org.jivesoftware.smack.PacketInterceptor;
import org.jivesoftware.smack.SmackConfiguration;
import org.jivesoftware.smack.XMPPConnection;
import org.jivesoftware.smack.XMPPException;
import org.jivesoftware.smack.filter.PacketFilter;
import org.jivesoftware.smack.filter.PacketTypeFilter;
import org.jivesoftware.smack.packet.Message;
import org.jivesoftware.smack.packet.Packet;
import org.jivesoftware.smack.packet.Presence;
import org.jivesoftware.smackx.filetransfer.FileTransfer;
import org.jivesoftware.smackx.filetransfer.FileTransfer.Status;
import org.jivesoftware.smackx.filetransfer.FileTransferManager;
import org.jivesoftware.smackx.filetransfer.OutgoingFileTransfer;
import org.jivesoftware.smackx.muc.DiscussionHistory;
import org.jivesoftware.smackx.muc.MultiUserChat;
import org.jivesoftware.smackx.packet.VCard;
import org.objectweb.petals.binding.xmpp.XmppBCImpl;
import org.objectweb.petals.binding.xmpp.util.JabberAvatarExtension;
import org.objectweb.petals.binding.xmpp.util.VCardUpdateExtension;
import org.objectweb.petals.binding.xmpp.util.XmppUtil;
import org.objectweb.petals.component.framework.PEtALSCDKException;
import org.objectweb.petals.component.framework.util.Exchange;
import org.objectweb.petals.component.framework.util.Extensions;
import org.objectweb.petals.component.framework.util.StringHelper;

/**
 * This manager is used to manage connections with the jabber server. All active
 * connections are stored in a map
 * 
 * @author mcarpentier - EBM Websourcing
 * 
 */
public class XmppSessionManager {

    protected Logger log;

    protected Map<String, SessionDescriptor> xmppSessions = new HashMap<String, SessionDescriptor>();

    /**
     * files destination directory
     */
    protected String destDir;

    protected final int TRANSFERT_CHECK_DELAY = 500;

    /**
     * @param destDir the default files destination dir
     * @param log
     *            the component logger
     */
    public XmppSessionManager(final Logger log, final String destDir) {
        super();
        this.log = log;
        this.destDir = destDir;
    }

    /**
     * 
     * Init or restart a timer that will disconnect the user on inactivity
     * usually called after user has performed an action
     * 
     * @param xmppSession
     *            the {@link SessionDescriptor} to manage inactivity
     */
    public final void manageInactivity(final SessionDescriptor xmppSession) {
        if (xmppSession.getInactivityChecker() != null) {
            xmppSession.getInactivityChecker().restart();
        } else {
            // init a timer that will disconnect the user on inactivty
            final long timeout = Long
                    .parseLong(xmppSession.getSessionTimeout());
            final InactivityChecker checker = new InactivityChecker(
                    xmppSession, timeout, this.log);
            xmppSession.setInactivityChecker(checker);
            checker.start();
        }
    }

    /**
     * Check whether an user already owns a connection or not
     * 
     * @param extensions
     *            {@link Extensions} with all required properties
     * @return the XMPP Connection {@link XMPPConnection}
     * @throws JBIException
     *             if an error occurs
     */
    public final SessionDescriptor retrieveXmppSessionFromExtensions(
            final Extensions extensions) throws JBIException {

        // retrieve user's JID from SU descriptor
        final Properties extensionsProps = extensions.getPetalsExtensions();
        final String username = extensionsProps
                .getProperty(XmppBCImpl.USERNAME_PATHELEMENT);
        final String resource = extensionsProps
                .getProperty(XmppBCImpl.RESOURCE_PATHELEMENT);
        final String host = extensionsProps
                .getProperty(XmppBCImpl.HOSTNAME_PATHELEMENT);
        String tmpUsername = "";
        String tmpHost = "";
        String tmpResource = "";
        if (!(StringHelper.isNullOrEmpty(username))) {
            tmpUsername = username;
        }
        if (!(StringHelper.isNullOrEmpty(host))) {
            tmpHost = "@" + host;
        }
        if (!(StringHelper.isNullOrEmpty(resource))) {
            tmpResource = "/" + resource;
        }
        final String jid = tmpUsername + tmpHost + tmpResource;

        // retrieve user's connection if exists
        final SessionDescriptor sess = this.retrieveSessionIfExists(jid);

        return sess;
    }

    /**
     * Retrieve user's {@link XMPPConnection} if already connected
     * @param jid the JID of the user
     * @return the XMPP Connection {@link XMPPConnection}
     */
    public final SessionDescriptor retrieveSessionIfExists(final String jid) {
        SessionDescriptor result = null;
        if (this.xmppSessions.containsKey(jid)) {
            result = this.xmppSessions.get(jid);
        }
        return result;

    }

    /**
     * Log to the server and send a presence packet to notify your connection
     * 
     * @param xmppConnection
     *            the {@link XMPPConnection} of the user
     * @param login
     *            the login of the user
     * @param password
     *            the login of the user
     * @param resource
     *            the resource used by the user for this connection
     * 
     * @throws JBIException
     *             if login fail
     */
    public final void xmppLogin(final XMPPConnection xmppConnection,
            final String login, final String password, final String resource)
            throws JBIException {

        try {
            if (!xmppConnection.isConnected()) {
                final String msg = "Not connected to server";
                throw new JBIException(msg);
            }
            if (!xmppConnection.isAuthenticated()) {
                if (StringHelper.isNullOrEmpty(login)) {
                    // Try to log in anonymously
                    xmppConnection.loginAnonymously();
                } else {
                    // log in to the server
                    this.log.info("Logging in to " + xmppConnection.getHost()
                            + ":" + xmppConnection.getPort() + " as " + login);

                    xmppConnection.login(login, password);

                }
            }
        } catch (final XMPPException e) {
            final String msg = "Error logging in to server";
            this.log.log(Level.SEVERE, msg, e);
            throw new JBIException(msg);
        }

    }

    /**
     * Send a text message to an user
     * 
     * @param xmppConnection
     *            the {@link XMPPConnection} of the user
     * @param xmppMessage
     *            the {@link Message} to send
     * 
     * @throws JBIException
     *             if send fails
     * 
     */
    public final void sendPrivateMessage(final XMPPConnection xmppConnection,
            final Message xmppMessage) throws JBIException {
        if ((!(xmppMessage == null))
                && (!StringHelper.isNullOrEmpty(xmppMessage.getBody()))
                && (!StringHelper.isNullOrEmpty(xmppMessage.getTo()))) {

            // send message via connection's chat manager
            final ChatManager chatmanager = xmppConnection.getChatManager();
            final Chat newChat = chatmanager.createChat(xmppMessage.getTo(),
                    null);

            try {

                newChat.sendMessage(xmppMessage);
            } catch (final XMPPException e) {
                this.log.log(Level.SEVERE, e.getMessage(), e);
                throw new JBIException(e.getMessage());
            }
        } else {
            final String msg = "Error : incorrect message : destination and content must be specified";
            throw new JBIException(msg);
        }
    }

    /**
     * Send a text message to a room Join the room if needed
     * 
     * @param xmppConnection
     *            the {@link XMPPConnection} of the user
     * @param sessionDescriptor the session descriptor for the user
     * @param xmppMessage
     *            the {@link Message} to send
     * @param roomPassword
     *            the password of the room
     * 
     * @throws JBIException
     *             if send fails
     * 
     */
    public final void sendRoomMessage(final XMPPConnection xmppConnection,
            final SessionDescriptor sessionDescriptor,
            final Message xmppMessage, final String roomPassword)
            throws JBIException {
        if ((!(xmppMessage == null))
                && (!StringHelper.isNullOrEmpty(xmppMessage.getBody()))
                && (!StringHelper.isNullOrEmpty(xmppMessage.getTo()))) {

            try {
                // HashMap<String, MultiUserChat> rooms = new HashMap<String,
                // MultiUserChat>();
                Map<String, MultiUserChat> rooms = sessionDescriptor.getRooms();
                final String room = xmppMessage.getTo();

                MultiUserChat muc = null;
                if (rooms != null) {
                    if (rooms.get(room) != null) {
                        muc = rooms.get(room);
                    }
                }
                if (muc == null) {
                    muc = new MultiUserChat(xmppConnection, room);

                    // join the room
                    // avoid the server to send history
                    final DiscussionHistory history = new DiscussionHistory();
                    history.setMaxStanzas(0);
                    muc.join(sessionDescriptor.getUser().getUsername(),
                            roomPassword, history, SmackConfiguration
                                    .getPacketReplyTimeout());

                    // add newly joined room to user's rooms
                    if (sessionDescriptor.getRooms() == null) {
                        sessionDescriptor
                                .setRooms(new HashMap<String, MultiUserChat>());
                    }
                    rooms = sessionDescriptor.getRooms();
                    rooms.put(room, muc);

                }

                // send message to the room
                muc.sendMessage(xmppMessage);

            } catch (final XMPPException e) {
                final String msg = e.getMessage();
                this.log.log(Level.SEVERE, msg, e);
                throw new JBIException(msg);
            }
        } else {
            final String msg = "Error : incorrect message - destination and content must be specified";
            throw new JBIException(msg);
        }
    }

    /**
     * Send all message attachments as files multiple attachments is not
     * supported yet
     * @param SessionDescriptor the user's {@link SessionDescriptor}
     * @param nMessage
     *            the {@link NormalizedMessage} received
     * @param message
     *            the {@link Message} containing the file(s) to send.
     * 
     * @throws JBIException
     *             if an error occurs during file transfert
     * 
     */
    public final void sendAttachmentsAsFiles(final SessionDescriptor sd,
            final NormalizedMessage nMessage, final Message message)
            throws JBIException {
        final Set attachIds = nMessage.getAttachmentNames();
        Set<File> files = new HashSet<File>();

        // retrieve all attachments and write corresponding temp files to disk
        if (!(attachIds.isEmpty() && !(attachIds == null))) {
            files = XmppUtil.getInstance().writeMessageAttachments(nMessage,
                    this.destDir);
        }
        try {

            for (File f : files) {

                if (StringHelper.isNullOrEmpty(message.getTo())) {
                    final String err = "The destination address must be specified";
                    this.log.severe(err);
                    throw new PEtALSCDKException(err);
                }

                // Create the file transfer manager
                final FileTransferManager manager = new FileTransferManager(sd
                        .getXmppConnection());

                // Create the outgoing file transfer
                final OutgoingFileTransfer transfer = manager
                        .createOutgoingFileTransfer(message.getTo());

                this.log.info("starting sending file..." + f.getName());
                transfer.sendFile(f, message.getBody());

                // loop until file transfert is over
                boolean flag = true;
                long time = System.currentTimeMillis();
                long written = transfer.getAmountWritten();
                while (flag) {
                    Thread.sleep(this.TRANSFERT_CHECK_DELAY);
                    final FileTransfer.Status status = transfer.getStatus();
                    if ((status == Status.error) || (status == Status.complete)
                            || (status == Status.cancelled)
                            || (status == Status.refused)) {
                        flag = false;
                    }
                    // each 20 secs, check if transfer has been initiated and is
                    // still running without any pb
                    if (System.currentTimeMillis() > time
                            + Long.parseLong(sd.getTransfertTimeout())) {
                        if (written == transfer.getAmountWritten()) {
                            transfer.cancel();
                            flag = false;
                        } else {
                            time = System.currentTimeMillis();
                            written = transfer.getAmountWritten();
                        }

                    }

                }

                // file transfert is over : look up it's status
                final FileTransfer.Status status = transfer.getStatus();

                if (status == Status.error) {
                    final String msg = 
                        "An error has occured during file transert - receiver may be unreachable";
                    throw new JBIException(msg);
                } else if (status == Status.cancelled) {
                    final String msg = "An error has occured - file transfer has been cancelled";
                    throw new JBIException(msg);
                } else if (status == Status.refused) {
                    final String msg = "Error - user has refused the file transfert request";
                    throw new JBIException(msg);
                } else if (status == Status.complete) {
                    this.log.info("file " + f.getName()
                            + "has been successfully sent");
                } else {
                    final String msg = "An error has occured during file transfert";
                    throw new JBIException(msg);
                }

                try {
                    f.delete();
                } catch (final SecurityException e) {
                    // file could'nt be deleted
                    this.log.severe("unable to delete temp file "
                            + f.getAbsoluteFile());
                }

            }
        } catch (final InterruptedException e) {
            final String msg = "An error has occured during the file transfert";
            this.log.severe(msg);
            throw new JBIException(msg);
        } catch (final XMPPException e) {
            final String msg = "An error has occured during the file transfert negociation";
            this.log.severe(msg);
            throw new JBIException(msg);
        }
    }

    /**
     * 
     * process incoming JBI message : send it as a file or as a text message
     * 
     * @param sessionDescriptor
     *            the {@link SessionDescriptor} of the connection
     * @param exchange
     *            the {@link MessageExchangeWrapper}
     * @param message
     *            {@link Message} to process
     * @throws JBIException
     *             if an error occurs during process
     */
    public final void processMessage(final SessionDescriptor sessionDescriptor,
            final Exchange exchange, final Message message) throws JBIException {
        final NormalizedMessage nMsg = exchange.getInMessage();
        final Set files = nMsg.getAttachmentNames();

        // user has sent a message - restart or init inactivity timer
        if (sessionDescriptor.getUser() != null) {
            this.manageInactivity(sessionDescriptor);
        }

        if ((files == null) || (files.isEmpty())) {
            // no files to send, send body as text

            // for text messages, only IN_ONLY mep is supported
            if (!(exchange.isInOnlyPattern())) {
                final String msg = "Only IN_ONLY Message Exchange Pattern is supported";
                throw new JBIException(msg);
            }

            // check whether the message is destinated to a room or not
            final String room = (String) exchange
                    .getProperty(XmppBCImpl.DESTINATION_ROOM);
            if (StringHelper.isNullOrEmpty(room)) {
                // send message to an user
                this.sendPrivateMessage(sessionDescriptor.getXmppConnection(),
                        message);
            } else {
                // send message to a room
                final String pwd = (String) exchange
                        .getProperty(XmppBCImpl.ROOM_PASSWORD);
                message.setTo(room);
                this.sendRoomMessage(sessionDescriptor.getXmppConnection(),
                        sessionDescriptor, message, pwd);
            }

        } else if (files.size() > 1) {
            final String err = "Multiple attchements is not supported";
            this.log.severe(err);
            throw new JBIException(err);
        } else {
            // Send message's attachment
            this.sendAttachmentsAsFiles(sessionDescriptor, exchange
                    .getInMessage(), message);

            // no error detected, file has been sent successfully
            // set out message if pattern isnt in only
            if (!(exchange.isInOnlyPattern())) {
                exchange.setOutMessageContent("file sent successfully");
            }

        }

    }

    /**
     * Check whether user is already connected or not ; if not, create a new
     * {@link XMPPConnection} initialized with the given properties
     * 
     * @param sessionDescriptor
     *            a {@link SessionDescriptor} the session to connect
     * @throws JBIException
     *             if an error occurs
     */
    public final void connectSession(final SessionDescriptor sessionDescriptor)
            throws JBIException {

        XMPPConnection connection = sessionDescriptor.getXmppConnection();
        final SessionUser user = sessionDescriptor.getUser();

        try {

            if (connection == null) {
                // init a new connection

                final String hostname = sessionDescriptor.getHostname();
                final String port = sessionDescriptor.getPort();

                final ConnectionConfiguration config = new ConnectionConfiguration(
                        hostname, Integer.parseInt(port));
                config.setCompressionEnabled(Boolean
                        .getBoolean("test.compressionEnabled"));
                config.setSocketFactory(null);
                config.setDebuggerEnabled(false);
                connection = new XMPPConnection(config);

                sessionDescriptor.setXmppConnection(connection);

            }

            // connect to server
            if (!(connection.isConnected())) {
                connection.connect();
            }

            // authenticate with server
            if ((user != null) && (!(connection.isAuthenticated()))) {
                this.xmppLogin(connection, user.getUsername(), user
                        .getPassword(), user.getResource());
            }

        } catch (final XMPPException e) {
            final String msg = e.getMessage();
            // disconnect from server if connected
            if ((connection != null) && (connection.isConnected())) {
                connection.disconnect();
            }
            this.log.log(Level.SEVERE, msg, e);
            throw new JBIException(msg);
        }

    }

    public final Map<String, SessionDescriptor> getXmppSessions() {
        return this.xmppSessions;
    }

    /**
     * 
     * Deals with VCard information
     * 
     * @param sessionDescriptor
     *            the {@link SessionDescriptor} of the xmpp connection
     * @throws PEtALSCDKException
     *             if an error occurs during VCard's operations
     */
    public final void handleVCard(final SessionDescriptor sessionDescriptor)
            throws PEtALSCDKException {
        if ((sessionDescriptor.getUser() != null)
                && (sessionDescriptor.getUser().getVCard() != null)) {
            final VCard myVCard = sessionDescriptor.getUser().getVCard();

            this.checkVCardUpdate(myVCard, sessionDescriptor
                    .getXmppConnection());

            this.enableVCardSend(sessionDescriptor);

            // send a Presence packet
            sessionDescriptor.getXmppConnection().sendPacket(
                    new Presence(Presence.Type.available));

        }
    }

    /**
     * Enable VCard support
     * 
     * @param sessionDescriptor
     *            the {@link SessionDescriptor} of the xmpp connection
     */
    private void enableVCardSend(final SessionDescriptor sessionDescriptor) {
        final PacketFilter presenceFilter = new PacketTypeFilter(Presence.class);
        final XMPPConnection connection = sessionDescriptor.getXmppConnection();
        final VCard vCard = sessionDescriptor.getUser().getVCard();

        connection.addPacketWriterInterceptor(new PacketInterceptor() {
            public void interceptPacket(final Packet packet) {
                final Presence newPresence = (Presence) packet;

                final VCardUpdateExtension update = new VCardUpdateExtension();
                final JabberAvatarExtension jax = new JabberAvatarExtension();

                final byte[] bytes = vCard.getAvatar();
                if (bytes != null) {
                    final String hash = vCard.getAvatarHash();
                    update.setPhotoHash(hash);
                    jax.setPhotoHash(hash);
                    // append VCard information to presence packets
                    newPresence.addExtension(update);
                    newPresence.addExtension(jax);
                }

            }
        }, presenceFilter);

    }

    /**
     * 
     * Check if user's local avatar is different form previously registered
     * avatar on server ; if different, save local avatar
     * 
     * @param myVCard
     *            user's Vcard
     * @param connection
     *            currenct xmpp connection
     * @throws 
     */
    public final void checkVCardUpdate(final VCard myVCard,
            final XMPPConnection connection) throws PEtALSCDKException {

        final String myVCardHash = myVCard.getAvatarHash();

        if (!(StringHelper.isNullOrEmpty(myVCardHash))) {
            final VCard serverVCard = new VCard();
            try {
                serverVCard.load(connection);
                final String serverVCardHash = serverVCard.getAvatarHash();

                if ((StringHelper.isNullOrEmpty(serverVCardHash))
                        || (!(serverVCardHash.equalsIgnoreCase(myVCardHash)))) {

                    myVCard.save(connection);

                }

            } catch (final XMPPException e) {
                final String msg = "Error while saving user's VCard";
                this.log.severe(msg);
                throw new PEtALSCDKException(msg);
            }

        }
    }

}
