/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.binding.xmpp.util;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.StringReader;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.HashSet;
import java.util.Set;

import javax.activation.DataHandler;
import javax.jbi.JBIException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.objectweb.petals.component.framework.util.StringHelper;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * Some usefull functions to process XMPP messages
 * 
 * @author mcarpentier - EBM Websourcing
 * 
 */
public class XmppUtil {

    private static XmppUtil INSTANCE;

    public static XmppUtil getInstance() {
        if (INSTANCE == null) {
            INSTANCE = new XmppUtil();
        }
        return INSTANCE;
    }

    /**
     * Write all attachments to files. One file per attachment. The file will
     * have the name of the attachment.
     * 
     * @param message
     *            the message to get attachment from
     * @param destDir
     *            the path where the files will be written
     * @throws FileTransferBCException
     */
    public Set<File> writeMessageAttachments(final NormalizedMessage message,
            final String destDir) throws JBIException {

        Set attachIds = message.getAttachmentNames();
        Set<File> files = new HashSet<File>();
        for (Object name : attachIds) {
            DataHandler handler = message.getAttachment((String) name);
            FileOutputStream fos = null;

            try {
                File f = new File(destDir);
                if (!f.exists()) {
                    f.mkdirs();
                } else {
                    f.delete();
                }

                File outFile = new File(f.getAbsolutePath(), (String) name);

                fos = new FileOutputStream(new File(outFile.getAbsoluteFile()
                        + ""));

                handler.writeTo(fos);
                fos.flush();
                files.add(outFile);

            } catch (Exception e) {
                throw new JBIException("Error: writing attachment "
                        + handler.getName() + "  " + e.getMessage());
            } finally {
                try {
                    fos.close();
                } catch (Exception e) {
                }
            }
        }
        return files;
    }

    /**
     * Remove the root tag in the XML string XML string must be correctly
     * formatted
     * 
     * @param body
     *            the xml text to wrap
     * @return the xml string wrapped if possible
     * 
     */
    public String deleteRootTag(final String body) {
        String result = body;
        if (!(StringHelper.isNullOrEmpty(body))) {
            int a = result.lastIndexOf("</");
            int b = result.indexOf(">");
            result = body.substring(b + 1, a);
        }

        return result;
    }

    /**
     * Delete root tag from an XML string
     * 
     * @param body
     *            an xml string to analyze
     * @param tag
     *            an empty string to put the foud root tag
     * @return
     */
    public String deleteRootTagIfExists(final String body, String tag) {

        int b1 = body.lastIndexOf("</");
        int b2 = body.lastIndexOf(">");
        int a1 = body.indexOf("<");
        int a2 = body.indexOf(">");

        // check if there's a well formed root tag
        if ((b1 != 0) || (a1 == -1) || (b1 == -1) || (a2 == -1) || (b2 == -1)) {
            tag = "";
            return body;
        }

        String firstTagText = body.substring(a1 + 1, a2);
        String lastTagText = body.substring(a2 + 1, b1);

        if ((StringHelper.isNullOrEmpty(firstTagText))
                || (StringHelper.isNullOrEmpty(lastTagText))) {
            tag = "";
            return body;
        }
        if (!(firstTagText.equalsIgnoreCase(lastTagText))) {
            tag = "";
            return body;
        }
        tag = firstTagText;
        return body.substring(a2 + 1, b1);

    }

    /**
     * Encode a string using {@link URLEncoder}
     * 
     * @param content
     *            the string to encode
     * @param format
     *            char format. e.g : UTF-8
     * @return the encoded string
     * @throws JBIException
     *             if an error occurs during encoding
     */
    public String encode(final String content, final String format) throws JBIException {
        try {
            return URLEncoder.encode(content, format);
        } catch (UnsupportedEncodingException e) {
            throw new JBIException("Error: unsupported encoding format ");
        }
    }

    /**
     * Decode a string using {@link URLEncoder}
     * 
     * @param content
     *            the string to decode
     * @param format
     *            char format. e.g : UTF-8
     * @return the decoded string
     * @throws JBIException
     *             if an error occurs during decoding
     */
    public String decode(final String content, final String format) throws JBIException {
        try {
            return URLDecoder.decode(content, format);
        } catch (UnsupportedEncodingException e) {
            throw new JBIException("Error: unsupported encoding format ");
        }
    }

    /**
     * @param body
     *            the string to add root tags
     * @param tag
     *            root tag to add to the string
     * @return the string with added root tags
     */
    public String addRootTag(final String body, final String tag) {
        String result = "<" + tag + ">" + body + "</" + tag + ">";
        return result;
    }

    /**
     * @param xml
     *            the xml string to analyze
     * @return true if the string is well formed.
     * @throws JBIException
     *             if an error occurs while analyzing string
     */
    public boolean isXmlWellFormed(final String xml) throws JBIException {
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory
                    .newInstance();
            factory.newDocumentBuilder();

            DocumentBuilder parser;
            parser = factory.newDocumentBuilder();
            InputSource is = new InputSource(new StringReader(xml));
            Document document = parser.parse(is);
        } catch (ParserConfigurationException e) {
            return false;
        } catch (SAXException e) {
            return false;
        } catch (IOException e) {
            String msg = "Error while analyzing XML String";
            e.printStackTrace();
            throw new JBIException(msg, e);
        }
        return true;
    }

}
